@@ -10,14 +10,14 @@ is usually 4K or 8K bytes.  From 1 to 3 bytes per page are
 preallocated in a table at compile time.  this must be a power of 2 if
 SGC is enabled.  */
 
-#define MAXPAGE 128*1024
-#define VSSIZE 128*1024
-#define BDSSIZE 2*1024
-#define IHSSIZE 4*1024
-#define FRSSIZE 4*1024
-#define CSSIZE  2*1024*1024
+#undef MAXPAGE
+#undef VSSIZE
+#undef BDSSIZE
+#undef IHSSIZE
+#undef FRSSIZE
+#undef CSSIZE
 
-#define HOLEPAGE (MAXPAGE/10)
+#undef HOLEDIV
 
 /* check to see if getcwd exists
 */
@@ -95,6 +95,13 @@ SGC is enabled.  */
 */
 
 #define CSTACK_ADDRESS 0
+#define CSTACK_DIRECTION 0
+#define CSTACK_ALIGNMENT 0
+#define MEM_TOP 0
+#define MEM_RANGE 0
+#define SHARED_LIB_HEAP_CEILING 0
+#undef IM_FIX_BASE
+#undef IM_FIX_LIM
 
 /* define if SIGSYS is defined in signal.h */
 
@@ -141,7 +148,7 @@ SGC is enabled.  */
 /* bfd support */
 #undef HAVE_LIBBFD
 #undef NEED_CONST
-#undef HAVE_BFD_BOOLEAN
+#define HAVE_BFD_BOOLEAN
 
 #ifdef HAVE_BFD_BOOLEAN
 #define MY_BFD_BOOLEAN bfd_boolean
@@ -159,7 +166,7 @@ SGC is enabled.  */
 
 #ifdef IN_NUM_CO
 #ifdef HAVE_ISNORMAL
-#define _GNU_SOURCE
+#define __USE_ISOC99 1
 #include <math.h>
 #define ISNORMAL(a) isnormal(a)
 #else
@@ -180,7 +187,7 @@ SGC is enabled.  */
 
 #ifdef NEED_ISFINITE
 #ifdef HAVE_ISFINITE
-#define _GNU_SOURCE
+#define __USE_ISOC99 1
 #include <math.h>
 #define ISFINITE(a) isfinite(a)
 #else
@@ -225,6 +232,10 @@ SGC is enabled.  */
 /*  #endif */
 
 #undef SIZEOF_LONG
+#undef SIZEOF_INT
+#undef SIZEOF_SHORT
+#undef SIZEOF_CHAR
+#undef CHAR_SIZE
 
 #undef USE_DLOPEN
 
@@ -252,3 +263,7 @@ SGC is enabled.  */
 #undef HOST_SYSTEM
 #undef GCL_GPROF_START
 #undef HZ
+
+#undef TYPE_BITS
+#undef OBJ_ALIGN
+#undef NEG_CSTACK_ADDRESS
