@@ -1,4 +1,4 @@
-(load (merge-pathnames "../clcs/clcs_macros.lisp" system:*system-directory*))
+(load (merge-pathnames "../clcs/gcl_clcs_macros.lisp" system:*system-directory*))
 
 (in-package "CONDITIONS")
 
