@@ -118,6 +118,7 @@ double object_to_double();
 /*  error.c  */
 EXTER object sKerror,sKparse_error;
 EXTER object sKwrong_type_argument;
+EXTER object sKcontrol_error;
 EXTER object sKcatch;
 EXTER object sKprotect;
 EXTER object sKcatchall;
@@ -301,7 +302,7 @@ EXTER object sLlambda_block;
 EXTER object sLlambda_closure;
 EXTER object sLlambda_block_closure;
 
-EXTER object sLfunction;
+EXTER object sLfunction,sLfunction_identifier;
 EXTER object sLmacro;
 EXTER object sLtag;
 EXTER object sLblock;
@@ -314,7 +315,6 @@ EXTER object sLblock;
 /*  number.c  */
 EXTER object shortfloat_zero;
 EXTER object longfloat_zero;
-#define make_fixnum(a) ((((FIXtemp=(a))+SMALL_FIXNUM_LIMIT)&(-2*SMALL_FIXNUM_LIMIT))==0?small_fixnum(FIXtemp):make_fixnum1(FIXtemp))
 object make_fixnum1(long);
 object make_ratio();
 object make_shortfloat();
@@ -597,16 +597,25 @@ EXTER object sSsetf_function;
 EXTER object sLcommon,sLnull,sLcons,sLlist,sLsymbol,sLarray,sLvector,sLbit_vector,sLstring;
 EXTER object sLsequence,sLsimple_array,sLsimple_vector,sLsimple_bit_vector,sLsimple_string;
 EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfloat,sLstring_char;
-EXTER object sLinteger,sLratio,sLshort_float,sLstandard_char,sLfixnum,sLpositive_fixnum, sLcomplex;
-EXTER object sLpositive_short,sLpositive_char;
+EXTER object sLinteger,sLratio,sLshort_float,sLstandard_char,sLinterpreted_function;
+
+EXTER object sLchar,sLnon_negative_char,sLnegative_char,sLsigned_char,sLunsigned_char;
+EXTER object sLshort,sLnon_negative_short,sLnegative_short,sLsigned_short,sLunsigned_short;
+EXTER object sLfixnum,sLnon_negative_fixnum,sLnegative_fixnum,sLsigned_fixnum,sLunsigned_fixnum;
+EXTER object sLlfixnum,sLnon_negative_lfixnum,sLnegative_lfixnum;
+EXTER object sLsigned_lfixnum,sLunsigned_lfixnum,sLnegative_bignum,sLnon_negative_bignum,sLbase_char;
+
+EXTER object sLseqind,sLrnkind;
+
+EXTER object sLcomplex;
 EXTER object sLsingle_float,sLpackage,sLbignum,sLrandom_state,sLdouble_float,sLstream,sLbit,sLreadtable;
 EXTER object sLlong_float,sLhash_table,sLstructure,sLboolean,sLfile_stream;
 
 #ifdef ANSI_COMMON_LISP
 /* new ansi types */
 EXTER object sLarithmetic_error,sLbase_char,sLbase_string,sLbroadcast_stream,sLbuilt_in_class;
-EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLcontrol_error,sLdivision_by_zero;
-EXTER object sLecho_stream,sLend_of_file,sLerror,sLextended_char,sLfile_error;
+EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLdivision_by_zero;
+EXTER object sLecho_stream,sLend_of_file,sLerror,sLextended_char,sLfile_error,sLcontrol_error;
 EXTER object sLfloating_point_inexact,sLfloating_point_invalid_operation,sLfloating_point_overflow;
 EXTER object sLfloating_point_underflow,sLgeneric_function,sLlogical_pathname,sLmethod,sLpackage_error;
 EXTER object sLparse_error,sLprint_not_readable,sLprogram_error,sLreader_error,sLserious_condition;
@@ -614,7 +623,7 @@ EXTER object sLsimple_base_string,sLsimple_condition,sLsimple_type_error,sLsimpl
 EXTER object sLstandard_generic_function,sLstandard_method,sLstandard_object,sLstorage_condition;
 EXTER object sLstream_error,sLstring_stream,sLstructure_class,sLstyle_warning,sLsynonym_stream;
 EXTER object sLtwo_way_stream,sLtype_error,sLunbound_slot,sLunbound_variable,sLundefined_function,sLwarning;
-EXTER object sLmethod_combination,sLstructure_object;
+EXTER object sLmethod_combination,sLstructure_object,sLdynamic_extent;
 #endif
 
 EXTER object sLsatisfies;
