@@ -20,7 +20,7 @@
 /* alloc.c:1000:OF */ extern void gcl_init_alloc_function (void); /* () */
 /* alloc.c:1126:OF */ extern void free (void *ptr); /* (ptr) void *ptr; */
 /* array.c:57:OF */ extern void Laref (void); /* () */
-/* array.c:126:OF */ extern object fLsvref (object x, ufixnum i); /* (x, i) object x; unsigned int i; */
+/* array.c:126:OF */ extern object fLsvref (object x, fixnum i); /* (x, i) object x; unsigned int i; */
 /* array.c:142:OF */ extern object fLrow_major_aref (object x,fixnum i); /* (x, i) object x; int i; */
 /* array.c:190:OF */ extern object fSaset1 (object x,fixnum i, object val); /* (x, i, val) object x; int i; object val; */
 /* array.c:262:OF */ extern void siLaset (void); /* () */
@@ -246,6 +246,7 @@ typedef void (*funcvoid)(void);
 #endif
 /* sgbc.c:1246:OF */ extern void memory_protect (int on); /* (on) int on; */
 /* sgbc.c:1306:OF */ extern void perm_writable (char *p, long n); /* (p, n) char *p; int n; */
+/* sgbc.c:1306:OF */ extern void un_perm_writable (char *p, long n); /* (p, n) char *p; int n; */
 /* sgbc.c:1321:OF */ extern void system_error (void); /* () */
 /* gbc.c:1357:OF */ extern void gcl_init_GBC (void); /* () */
 /* gnumalloc.c:286:OF */ extern void malloc_init (char *start, void (*warnfun) (/* ??? */)); /* (start, warnfun) char *start; void (*warnfun)(); */
@@ -296,7 +297,7 @@ typedef void (*funcvoid)(void);
 /* main.c:528:OF */ extern void bds_overflow (void); /* () */
 /* main.c:537:OF */ extern void frs_overflow (void); /* () */
 /* main.c:546:OF */ extern void ihs_overflow (void); /* () */
-/* main.c:556:OF */ extern void segmentation_catcher (int); /* () */
+/* main.c:556:OF */ extern void segmentation_catcher (int,long,void *,char *); /* () */
 /* main.c:587:OF */ extern void Lby (void); /* () */
 /* main.c:607:OF */ extern void Lquit(void); /* () */
 /* main.c:612:OF */ extern void Lexit(void); /* () */
@@ -503,6 +504,7 @@ typedef void (*funcvoid)(void);
 /* unixfsys.c:145:OF */ extern char *getwd (char *buffer); /* (buffer) char *buffer; */
 /* unixfsys.c:209:OF */ extern void coerce_to_filename (object pathname, char *p); /* (pathname, p) object pathname; char *p; */
 /* unixfsys.c:329:OF */ extern bool file_exists (object file); /* (file) object file; */
+/* unixfsys.c:359:OF */ extern FILE *fopen_not_dir (char *filename, char *option); /* (filename, option) char *filename; char *option; */
 /* unixfsys.c:359:OF */ extern FILE *backup_fopen (char *filename, char *option); /* (filename, option) char *filename; char *option; */
 /* unixfsys.c:372:OF */ extern int file_len (FILE *fp); /* (fp) FILE *fp; */
 /* unixfsys.c:382:OF */ extern object truename (object); /* () */
@@ -543,6 +545,7 @@ typedef void (*funcvoid)(void);
 
 
 /*  readline.d */
+extern int readline_on;
 void
 gcl_init_readline_function(void);
 
@@ -599,6 +602,9 @@ gcl_init_string_function(void);
 void
 gcl_init_symbol_function(void);
 
+void
+gcl_init_socket_function(void);
+
 void
 gcl_init_hash(void);
 
@@ -1764,6 +1770,23 @@ gcl_isnormal_double(double);
 int 
 gcl_isnormal_float(float);
 
+void wipe_stack   (VOL void *);
+void clear_c_stack(VOL unsigned);
+
+long
+opt_maxpage(struct typemanager *);
+
+typedef MP_INT * GEN;
+
+MP_INT *
+otoi(object);
+
+MP_INT *
+stoi(fixnum);
+
+object
+read_byte1(object,object);
+
 #ifdef SGC
 void
 memprotect_test_reset(void);
@@ -1790,3 +1813,7 @@ int sigprocmask ( int how, const sigset_t *set, sigset_t *oldset );
 void
 gprof_cleanup(void);
 #endif
+
+
+unsigned long
+ihash_equal1(object,int);
