@@ -36,7 +36,7 @@ DEFCONST("ARRAY-RANK-LIMIT", sLarray_rank_limit, LISP,
 	 make_fixnum(ARRAY_RANK_LIMIT),"");
 
 DEFCONST("ARRAY-DIMENSION-LIMIT", sLarray_dimension_limit,
-	 LISP, make_fixnum(MOST_POSITIVE_FIX),"");
+	 LISP, make_fixnum(MOST_POSITIVE_FIX>>3),"");
 DEFCONST("ARRAY-TOTAL-SIZE-LIMIT", sLarray_total_size_limit,
 	 LISP, make_fixnum(MOST_POSITIVE_FIX),"");
 
@@ -63,7 +63,7 @@ extern short aet_sizes[];
 #define N_FIXNUM_ARGS 6
 
 DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
-       NONE, OO, II, II, II,void,Laref,(object x,fixnum i, ...),"")
+       ONE_VAL, OO, II, II, II,void,Laref,(object x,fixnum i, ...),"")
 { int n = VFUN_NARGS;
   int i1;
   va_list ap;
@@ -115,7 +115,7 @@ fScheck_bounds_bounds(object x, int i)
 }
 
 DEFUNO_NEW("SVREF", object, fLsvref, LISP, 2, 2,
-      ONE_VAL, OO, IO, OO,OO,void,Lsvref,(object x,ufixnum i),
+      ONE_VAL, OO, IO, OO,OO,void,Lsvref,(object x,fixnum i),
       "For array X and index I it returns (aref x i) ")
 {
  if (type_of(x)==t_vector
@@ -127,8 +127,8 @@ DEFUNO_NEW("SVREF", object, fLsvref, LISP, 2, 2,
  return(Cnil);
 }
     
-DEFUN_NEW("ROW-MAJOR-AREF", object, fLrow_major_aref, LISP, 2, 2,
-       NONE, OO, IO, OO,OO,(object x,fixnum i),
+DEFUNO_NEW("ROW-MAJOR-AREF", object, fLrow_major_aref, LISP, 2, 2,
+       NONE, OO, IO, OO,OO,void,Lrow_major_aref,(object x,fixnum i),
       "For array X and index I it returns (aref x i) as if x were \
 1 dimensional, even though its rank may be bigger than 1")
 {
@@ -146,16 +146,19 @@ DEFUN_NEW("ROW-MAJOR-AREF", object, fLrow_major_aref, LISP, 2, 2,
       i += BV_OFFSET(x);
       return make_fixnum(BITREF(x, i));
     case aet_fix:
+    case aet_nnfix:
       return make_fixnum(x->fixa.fixa_self[i]);
     case aet_sf:
       return make_shortfloat(x->sfa.sfa_self[i]);
     case aet_lf:
       return make_longfloat(x->lfa.lfa_self[i]);
     case aet_char:
+    case aet_nnchar:
       return small_fixnum(x->st.st_self[i]);
     case aet_uchar:
       return small_fixnum(x->ust.ust_self[i]);
     case aet_short:
+    case aet_nnshort:
       return make_fixnum(SHORT_GCL(x, i));
     case aet_ushort:
       return make_fixnum(USHORT_GCL(x, i));
@@ -209,6 +212,7 @@ DEFUN_NEW("ASET1", object, fSaset1, SI, 3, 3, NONE, OO, IO, OO,OO,(object x, fix
 	     goto AGAIN_BIT;}
        break;}
     case aet_fix:
+    case aet_nnfix:
       ASSURE_TYPE(val,t_fixnum);
       (x->fixa.fixa_self[i]) = Mfix(val);
       break;
@@ -221,6 +225,7 @@ DEFUN_NEW("ASET1", object, fSaset1, SI, 3, 3, NONE, OO, IO, OO,OO,(object x, fix
       (x->lfa.lfa_self[i]) = Mlf(val);
       break;
     case aet_char:
+    case aet_nnchar:
       ASSURE_TYPE(val,t_fixnum);
       x->st.st_self[i] = Mfix(val);
       break;
@@ -229,6 +234,7 @@ DEFUN_NEW("ASET1", object, fSaset1, SI, 3, 3, NONE, OO, IO, OO,OO,(object x, fix
       (x->ust.ust_self[i])= Mfix(val);
       break;
     case aet_short:
+    case aet_nnshort:
       ASSURE_TYPE(val,t_fixnum);
       SHORT_GCL(x, i) = Mfix(val);
       break;
@@ -434,24 +440,27 @@ fSmake_vector1_1(fixnum n,fixnum elt_type,object staticp) {
 }
 
 
-static object DFLT_aet_object = Cnil;	
+static object DFLT_aet_object;	
 static char DFLT_aet_ch = ' ';
 static char DFLT_aet_char = 0; 
 static fixnum DFLT_aet_fix = 0  ;		
 static short DFLT_aet_short = 0;
 static shortfloat DFLT_aet_sf = 0.0;
 static longfloat DFLT_aet_lf = 0.0;	
-static object Iname_t = sLt;
+static object Iname_t;
 static struct { char * dflt; object *namep;} aet_types[] =
 {   {(char *)	&DFLT_aet_object,	&Iname_t,},	/*  t  */
     {(char *)	&DFLT_aet_ch, &sLstring_char,},/*  string-char  */
     {(char *)	&DFLT_aet_fix, &sLbit,},		/*  bit  */
     {(char *)	&DFLT_aet_fix,	&sLfixnum,}, 	/*  fixnum  */
+    {(char *)	&DFLT_aet_fix,	&sLnon_negative_fixnum,}, 	/*  non-neg fixnum  */
     {(char *)	&DFLT_aet_sf, &sLshort_float,},			/*  short-float  */
     {(char *)	&DFLT_aet_lf, &sLlong_float,},	/*  long-float  */
     {(char *)	&DFLT_aet_char,&sLsigned_char,},               /* signed char */
-    {(char *)    &DFLT_aet_char,&sLunsigned_char,},               /* unsigned char */
+    {(char *)	&DFLT_aet_char,&sLnon_negative_char,},               /* non-neg char */
+    {(char *)   &DFLT_aet_char,&sLunsigned_char,},               /* unsigned char */
     {(char *)	&DFLT_aet_short,&sLsigned_short,},              /* signed short */
+    {(char *)	&DFLT_aet_short,&sLnon_negative_short,},              /* non-neg short */
     {(char *)	&DFLT_aet_short, &sLunsigned_short},    /*  unsigned short   */
 	};
 
@@ -460,13 +469,13 @@ DEFUN_NEW("GET-AELTTYPE",object,fSget_aelttype,SI,1,1,NONE,OO,OO,OO,OO,(object x
   for (i=0 ; i <   aet_last ; i++)
     if (x == * aet_types[i].namep)
       return make_fixnum((enum aelttype) i);
-  if (x == sLlong_float || x == sLsingle_float || x == sLdouble_float)
+  if (x == sLsingle_float || x == sLdouble_float)
     return make_fixnum(aet_lf);
-  if (x==sLpositive_char)
+  if (x==sLnegative_char)
     return make_fixnum(aet_char);
-  if (x==sLpositive_short)
+  if (x==sLnegative_short)
     return make_fixnum(aet_short);
-  if (x==sLpositive_fixnum)
+  if (x==sLnegative_fixnum || x==sLsigned_fixnum)
     return make_fixnum(aet_fix);
   return make_fixnum(aet_object);
 }
@@ -536,7 +545,7 @@ DEFUN_NEW("MAKE-ARRAY1",object,fSmake_array1,SI,6,6,
     x->a.a_self = 0;
     x->a.a_rank = rank;
     x->a.a_displaced = Cnil;
-    x->a.a_dims = AR_ALLOC(alloc_relblock,rank,int);
+    x->a.a_dims = AR_ALLOC(alloc_relblock,rank,fixnum);
     i = 0;
     v = dimensions;
     while (i < rank)
@@ -754,12 +763,13 @@ Iarray_element_type(object x)
 /* add diff to body of x and arrays diisplaced to it */
 
 void
-adjust_displaced(object x, long diff)
-{
-	if (x->ust.ust_self != NULL)
-		x->ust.ust_self = (char *)((long)(x->a.a_self) + diff);
-	for (x = Mcdr(x->ust.ust_displaced);  x != Cnil;  x = Mcdr(x))
-		adjust_displaced(Mcar(x), diff);
+adjust_displaced(object x, long diff) {
+
+  if (x->ust.ust_self != NULL)
+    x->ust.ust_self = (unsigned char *)((long)(x->a.a_self) + diff);
+  for (x = Scdr(x->ust.ust_displaced);  x != Cnil;  x = Scdr(x)) 
+    adjust_displaced(Mcar(x), diff);
+  
 }
 
 
@@ -797,6 +807,7 @@ raw_aet_ptr(object x, short int typ)
     u.i=-Mfix(x);
     break;
   case aet_fix:    
+  case aet_nnfix:    
     /* STORE_TYPED(&u,fixnum, Mfix(x)); */
     u.i=Mfix(x);
     break;
@@ -809,6 +820,7 @@ raw_aet_ptr(object x, short int typ)
     u.d=Mlf(x);
     break;
   case aet_char:   
+  case aet_nnchar:   
     /* STORE_TYPED(&u, char, Mfix(x)); */
     u.c=(char)Mfix(x);
     break;
@@ -817,6 +829,7 @@ raw_aet_ptr(object x, short int typ)
     u.uc=(unsigned char)Mfix(x);
     break;
   case aet_short:  
+  case aet_nnshort:  
     /* STORE_TYPED(&u, short, Mfix(x)); */
     u.s=(short)Mfix(x);
     break;
@@ -856,12 +869,12 @@ gset(void *p1, void *val, int n, int typ)
     case aet_ch:     GSET(p1,n,char,val);
       /* Note n is number of fixnum WORDS for bit */
     case aet_bit:    GSET(p1,n,fixnum,val);
-    case aet_fix:    GSET(p1,n,fixnum,val);
+    case aet_fix:case aet_nnfix:    GSET(p1,n,fixnum,val);
     case aet_sf:     GSET(p1,n,shortfloat,val);
     case aet_lf:     GSET(p1,n,longfloat,val);
-    case aet_char:   GSET(p1,n,char,val);
+    case aet_char:case aet_nnchar:   GSET(p1,n,char,val);
     case aet_uchar:  GSET(p1,n,unsigned char,val);
-    case aet_short:  GSET(p1,n,short,val);
+    case aet_short:case aet_nnshort:  GSET(p1,n,short,val);
     case aet_ushort: GSET(p1,n,unsigned short,val);
     default:         FEerror("bad elttype",0);
     }
@@ -941,10 +954,12 @@ array_allocself(object x, int staticp, object dflt)
 		break;
 	case aet_ch:
 	case aet_char:
+	case aet_nnchar:
         case aet_uchar:
 		x->st.st_self = AR_ALLOC(*fun,n,char);
 		break;
         case aet_short:
+        case aet_nnshort:
         case aet_ushort:
 		x->ust.ust_self = (unsigned char *) AR_ALLOC(*fun,n,short);
 		break;
@@ -952,6 +967,7 @@ array_allocself(object x, int staticp, object dflt)
 		n = (n+W_SIZE-1)/W_SIZE;
 		SET_BV_OFFSET(x,0);
 	case aet_fix:
+	case aet_nnfix:
 		x->fixa.fixa_self = AR_ALLOC(*fun,n,fixnum);
 		break;
 	case aet_sf:
@@ -1038,7 +1054,24 @@ DEFUNO_NEW("ADJUSTABLE-ARRAY-P",object,fLadjustable_array_p,
        LISP,1,1,NONE,OO,OO,OO,OO,void,Ladjustable_array_p,(object x),"")
 { 
   IisArray(x);
-  return sLt;
+  switch (type_of(x)) {
+  case t_array:
+    x=x->a.a_adjustable ? Ct : Cnil; 
+    break;
+  case t_string:
+    x=x->st.st_adjustable ? Ct : Cnil; 
+    break;
+  case t_vector:
+    x=x->v.v_adjustable ? Ct : Cnil; 
+    break;
+  case t_bitvector:
+    x=x->bv.bv_adjustable ? Ct : Cnil; 
+    break;
+  default:
+    FEerror("Bad array type",0);
+    break;
+  }
+  return x;
 }
 
 DEFUNO_NEW("DISPLACED-ARRAY-P",object,fSdisplaced_array_p,SI,1,
@@ -1066,6 +1099,12 @@ DEFUNO_NEW("ARRAY-DIMENSION",object,fLarray_dimension,LISP,2,2,
    IisArray(x);
    return make_fixnum(x->v.v_dim);
 }
+#ifdef STATIC_FUNCTION_POINTERS
+object
+fLarray_dimension(object x,fixnum i) {
+  return FFN(fLarray_dimension)(x,i);
+}
+#endif
 
 static void
 Icheck_displaced(object displaced_list, object ar, int dim)
@@ -1509,6 +1548,9 @@ DEFUN_NEW("ASET-BY-CURSOR",object,fSaset_by_cursor,SI,3,3,
 void
 gcl_init_array_function(void) {
   make_function("ARRAY-DISPLACEMENT", Larray_displacement);
+  DFLT_aet_object=Cnil;
+  Iname_t=sLt;
+
 }
      
 
