@@ -169,6 +169,7 @@ DEFUNO_NEW("FSET",object,fSfset,SI
 	    type_of(function) == t_sfun ||
 	    type_of(function) == t_vfun ||
 	    type_of(function) == t_gfun ||
+	    type_of(function) == t_ifun ||
 	    type_of(function) == t_cclosure||
 	    type_of(function) == t_closure ||
 	    type_of(function) == t_afun 
@@ -238,27 +239,42 @@ DEFUNO_NEW("FMAKUNBOUND",object,fLfmakunbound,LISP
    ,1,1,NONE,OO,OO,OO,OO,void,Lfmakunbound,(object sym),"")
 
 {
-	/* 1 args */
-	if(type_of(sym) != t_symbol)
-		not_a_symbol(sym);
-	if (sym->s.s_sfdef != NOT_SPECIAL) {
-		if (sym->s.s_mflag) {
-			if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-				sym->s.s_sfdef = NOT_SPECIAL;
-		} else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-			FEerror("~S, a special form, cannot be redefined.",
-				1, sym);
-	}
-	remf(&(sym->s.s_plist),sStraced);
-	clear_compiler_properties(sym,Cnil);
-	if (sym->s.s_hpack == lisp_package &&
-	    sym->s.s_gfdef != OBJNULL && initflag) {
-		ifuncall2(sLwarn, make_simple_string(
-			"~S is being redefined."), sym);
-	}
-	sym->s.s_gfdef = OBJNULL;
-	sym->s.s_mflag = FALSE;
-	RETURN1(sym);
+
+  if(type_of(sym) != t_symbol && !setf_fn_form(sym)) 
+    TYPE_ERROR(sym,sLfunction_identifier);
+
+  /*FIXME -- store a symbol in plist for setf functions as opposed to
+    the function itself, and centralize function name resolution.
+    Allow for tracing, etc. thereby. 20050307 CM*/
+
+  if (setf_fn_form(sym)) {
+
+    if (get(sym->c.c_cdr->c.c_car,sSsetf_function,OBJNULL)!=OBJNULL)
+/*       FEundefined_function(sym); */
+      remf(&sym->c.c_cdr->c.c_car->s.s_plist,sSsetf_function);
+    RETURN1(sym);
+  }
+
+  /* 1 args */
+
+  if (sym->s.s_sfdef != NOT_SPECIAL) {
+    if (sym->s.s_mflag) {
+      if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+	sym->s.s_sfdef = NOT_SPECIAL;
+    } else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+      FEerror("~S, a special form, cannot be redefined.", 1, sym);
+  }
+  remf(&(sym->s.s_plist),sStraced);
+  clear_compiler_properties(sym,Cnil);
+  if (sym->s.s_hpack == lisp_package &&
+      sym->s.s_gfdef != OBJNULL && initflag) {
+    ifuncall2(sLwarn, make_simple_string("~S is being redefined."), sym);
+  }
+
+  sym->s.s_gfdef = OBJNULL;
+  sym->s.s_mflag = FALSE;
+  RETURN1(sym);
+
 }
 
 static void
@@ -330,8 +346,8 @@ setf(object place, object form)
 	  val = Ieval(form);
 	  return putprop(sym,val,key); 
 	}
-	if (fun == sLgetf) 
-	  Ieval(Mcaddr(args));
+/* 	if (fun == sLgetf)  */
+/* 	  Ieval(Mcaddr(args)); */
 	if (fun == sLaref) { f = siLaset; goto EVAL; }
 	if (fun == sLsvref) { f = siLsvset; goto EVAL; }
 	if (fun == sLelt) { f = siLelt_set; goto EVAL; }
