@@ -95,8 +95,8 @@ quick_call_sfun(object fun)
 { DEBUG_AVMA
   int i=fun->sfn.sfn_argd,n=SFUN_NARGS(i);
   enum ftype restype;
-  object *x,res,*base;
-  object *temp_ar=alloca(n*sizeof(object));
+  object *x,res=OBJNULL,*base;
+  object *temp_ar=ZALLOCA(n*sizeof(object));
 /*   i=fun->sfn.sfn_argd; */
 /*   n=SFUN_NARGS(i); */
   base = vs_base;
@@ -105,7 +105,7 @@ quick_call_sfun(object fun)
   restype = SFUN_RETURN_TYPE(i);
   SFUN_START_ARG_TYPES(i);
   /* for moment just support object and int */
-#define COERCE_ARG(a,type)  (type==f_object ? a : (object)(fix(a)))
+#define COERCE_ARG(a,type)  (type==f_object ? a : (type==f_fixnum ? (object)(fix(a)) : (object)otoi(a)))
   if (i==0)
     x=vs_base;
   else
@@ -117,8 +117,9 @@ quick_call_sfun(object fun)
   SET_TO_APPLY(res,fun->sfn.sfn_self,n,x);
   base[0]=
     (restype==f_object ?  res :
-     restype==f_fixnum ? make_fixnum((long)res)
-     :(object) (FEerror("Bad result type",0),Cnil));
+     (restype==f_fixnum ? make_fixnum((long)res) :
+      (restype==f_integer ? make_integer((GEN)res) :
+       (object) (FEerror("Bad result type",0),Cnil))));
   vs_base = base;
   vs_top=base+1;
   CHECK_AVMA;
@@ -157,13 +158,16 @@ call_vfun(object fun)
 void
 funcall(object fun)
 { 
-        object temporary;
-	object x;
-	 object * VOL top;
-	object *lex;
-	bds_ptr old_bds_top;
-	VOL bool b;
-	bool c;
+/*         object VOL sfirst=NULL; */
+/*         wipe_stack(&sfirst); */
+	{
+        object temporary=OBJNULL;
+	object x=OBJNULL;
+	object * VOL top=NULL;
+	object *lex=NULL;
+	bds_ptr old_bds_top=NULL;
+	VOL bool b=0;
+	bool c=0;
 	DEBUG_AVMA
       TOP:
 	if (fun == OBJNULL)
@@ -185,7 +189,7 @@ funcall(object fun)
 		return;
          case t_afun:
 	 case t_closure:
-	   { object res,*b = vs_base;
+	   { object res=OBJNULL,*b = vs_base;
 	     int n = vs_top - b;
 	     res = (object)IapplyVector(fun,n,b);
 	     n = fcall.nvalues;
@@ -197,7 +201,7 @@ funcall(object fun)
       case t_cclosure:
 
 	{
-		object *top, *base, l;
+		object *top=NULL, *base=NULL, l=OBJNULL;
 
 		if (fun->cc.cc_turbo != NULL) {
 			MMccall(fun, fun->cc.cc_turbo);
@@ -221,6 +225,13 @@ funcall(object fun)
 		FEundefined_function(fun);
 	      }
 
+	case t_ifun:
+	  {object x = fun->ifn.ifn_self;
+	  if (x) { fun = x; goto TOP;}
+	  else
+	    FEundefined_function(fun);
+	  }
+	  
 	case t_cons:
 		break;
 
@@ -338,7 +349,7 @@ END:
 	lex_env = lex;
 	if (not_pushed == 0) {ihs_pop();}
 	CHECK_AVMA;
-}}
+}}}
 
 void
 funcall_no_event(object fun)
@@ -353,7 +364,7 @@ funcall_no_event(object fun)
 
 	case t_cclosure:
 	{
-		object *top, *base, l;
+		object *top=NULL, *base=NULL, l=OBJNULL;
 
 		if (fun->cc.cc_turbo != NULL) {
 			(*fun->cc.cc_self)(fun->cc.cc_turbo);
@@ -894,7 +905,7 @@ EVAL_ARGS:
 	  top[0] = ans;
 	  vs_top = ++top;
 	  form = MMcdr(form);}
-	  n =top - base; /* number of args */
+	n =top - base; /* number of args */
 	if (Vapplyhook->s.s_dbind != Cnil) {
 	  base[0]= (object)n;
 	  base[0] = c_apply_n(list,n+1,base);
@@ -933,11 +944,11 @@ Ieval(object form)
 void
 eval(object form)
 { 
-        object temporary;
+        object temporary=OBJNULL;
         DEBUG_AVMA
-	object fun, x;
-	object *top;
-	object *base;
+	object fun=OBJNULL, x=OBJNULL;
+	object *top=NULL;
+	object *base=NULL;
 
 	cs_check(form);
 
@@ -1073,7 +1084,7 @@ EVAL_ARGS:
 	form = form->c.c_cdr;
 	base = vs_top;
 	top = vs_top;
-	while(!endp(form)) {
+	while(!endp_prop(form)) {
 		eval(MMcar(form));
 		top[0] = vs_base[0];
 		vs_top = ++top;
@@ -1129,17 +1140,21 @@ call_applyhook(object fun)
 	super_funcall(ah);
 }
 
-
 DEFUNO_NEW("FUNCALL",object,fLfuncall,LISP
-       ,1,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lfuncall,(object fun,...),"")
-{ va_list ap;
+       ,1,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lfuncall,(object fun,...),"") { 
+
+  va_list ap;
   object *new;
-  int n = VFUN_NARGS;
+  int i,n = VFUN_NARGS-1;
+
+  if (n>=65) FEerror("arg limit exceeded",0);
+  new=ZALLOCA(n*sizeof(*new));
+/*   wipe_stack(&n); */
   va_start(ap,fun);
-  {COERCE_VA_LIST(new,ap,n);
-  return IapplyVector(fun,n-1,new);
+  for (i=0;i<n;i++)
+    new[i]=va_arg(ap,object);
   va_end(ap);
- }
+  return IapplyVector(fun,n,new);
 }
 
 
@@ -1419,9 +1434,10 @@ fcalln_general(object first,va_list ap) {
       while (n-- > 0) {
 	typ= SFUN_NEXT_TYPE(i);
 	x =
-	  (typ==f_object ?	(jj ? va_arg(ap,object) : first):
-	   typ==f_fixnum ? make_fixnum((jj ? va_arg(ap,fixnum) : (fixnum)first)):
-	   (object) (FEerror("bad type",0),Cnil));
+	  (typ==f_object ?	(jj ? va_arg(ap,object) : first) :
+	   (typ==f_fixnum ? make_fixnum((jj ? va_arg(ap,fixnum) : (fixnum)first)) :
+	    (typ==f_integer ? make_integer((jj ? va_arg(ap,GEN) : (GEN)first)) :
+	     (object) (FEerror("bad type",0),Cnil))));
 	*(vs_top++) = x;
 	jj++;
       }
@@ -1438,8 +1454,9 @@ fcalln_general(object first,va_list ap) {
     vs_top=old_vs_top;
     /* vs_base=old_vs_base; */
     return (restype== f_object ? x :
-	    restype== f_fixnum ? (object) (fix(x)):
-	    (object) (FEerror("bad type",0),Cnil));
+	    (restype== f_fixnum ? (object) (fix(x)) :
+	     (restype== f_integer ? (object) (otoi(x)) :
+	      (object) (FEerror("bad type",0),Cnil))));
   }
 }
 
