@@ -44,7 +44,7 @@ static void
 fmt_roman(int,int,int,int,int);
 
 static void
-fmt_integer(object,bool,bool,int,int,int,int);
+fmt_integer(object,bool,bool,int,int,int,int,int);
 
 static void
 fmt_semicolon(bool,bool);
@@ -370,6 +370,11 @@ DIRECTIVE:
 	if (c == '@') {
 		atsign = TRUE;
 		c = ctl_advance();
+		if (!colon)
+		  if (c == ':') {
+		    colon = TRUE;
+		    c = ctl_advance();
+		  }
 	}
 	fmt_nparam = n;
 	switch (c) {
@@ -595,7 +600,7 @@ fmt_set_param(int i, int *p, int t, int v)
 static void
 fmt_ascii(bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object x;
 	int l, i;
 
@@ -633,7 +638,7 @@ fmt_ascii(bool colon, bool atsign)
 static void
 fmt_S_expression(bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object x;
 	int l, i;
 
@@ -671,59 +676,63 @@ fmt_S_expression(bool colon, bool atsign)
 static void
 fmt_decimal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0, commainterval=0;
 
-	fmt_max_param(3);
+	fmt_max_param(4);
 	fmt_set_param(0, &mincol, fmt_int, 0);
 	fmt_set_param(1, &padchar, fmt_char, ' ');
 	fmt_set_param(2, &commachar, fmt_char, ',');
+	fmt_set_param(3, &commainterval, fmt_int, 3);
 	fmt_integer(fmt_advance(), colon, atsign,
-		    10, mincol, padchar, commachar);
+		    10, mincol, padchar, commachar, commainterval);
 }
 
 static void
 fmt_binary(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0, commainterval=0;
 
-	fmt_max_param(3);
+	fmt_max_param(4);
 	fmt_set_param(0, &mincol, fmt_int, 0);
 	fmt_set_param(1, &padchar, fmt_char, ' ');
 	fmt_set_param(2, &commachar, fmt_char, ',');
+	fmt_set_param(3, &commainterval, fmt_int, 3);
 	fmt_integer(fmt_advance(), colon, atsign,
-		    2, mincol, padchar, commachar);
+		    2, mincol, padchar, commachar, commainterval);
 }
 
 static void
 fmt_octal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0, commainterval=0;;
 
-	fmt_max_param(3);
+	fmt_max_param(4);
 	fmt_set_param(0, &mincol, fmt_int, 0);
 	fmt_set_param(1, &padchar, fmt_char, ' ');
 	fmt_set_param(2, &commachar, fmt_char, ',');
+	fmt_set_param(3, &commainterval, fmt_int, 3);
 	fmt_integer(fmt_advance(), colon, atsign,
-		    8, mincol, padchar, commachar);
+		    8, mincol, padchar, commachar, commainterval);
 }
 
 static void
 fmt_hexadecimal(bool colon, bool atsign)
 {
-	int mincol, padchar, commachar;
+	int mincol=0, padchar=0, commachar=0, commainterval=0;;
 
-	fmt_max_param(3);
+	fmt_max_param(4);
 	fmt_set_param(0, &mincol, fmt_int, 0);
 	fmt_set_param(1, &padchar, fmt_char, ' ');
 	fmt_set_param(2, &commachar, fmt_char, ',');
+	fmt_set_param(3, &commainterval, fmt_int, 3);
 	fmt_integer(fmt_advance(), colon, atsign,
-		    16, mincol, padchar, commachar);
+		    16, mincol, padchar, commachar, commainterval);
 }
 
 static void
 fmt_radix(bool colon, bool atsign)
 {
-	int radix, mincol, padchar, commachar;
+	int radix=0, mincol=0, padchar=0, commachar=0, commainterval=0;;
 	object x;
 	int i, j, k;
 	int s, t;
@@ -740,7 +749,7 @@ fmt_radix(bool colon, bool atsign)
 				i = -1;
 			if ((!colon && (i <= 0 || i >= 4000)) ||
 			    (colon && (i <= 0 || i >= 5000))) {
-				fmt_integer(x, FALSE, FALSE, 10, 0, ' ', ',');
+				fmt_integer(x, FALSE, FALSE, 10, 0, ' ', ',', 3);
 				return;
 			}
 			fmt_roman(i/1000, 'M', '*', '*', colon);
@@ -787,22 +796,23 @@ fmt_radix(bool colon, bool atsign)
 		}
 		return;
 	}
-	fmt_max_param(4);
+	fmt_max_param(5);
 	fmt_set_param(0, &radix, fmt_int, 10);
 	fmt_set_param(1, &mincol, fmt_int, 0);
 	fmt_set_param(2, &padchar, fmt_char, ' ');
 	fmt_set_param(3, &commachar, fmt_char, ',');
+	fmt_set_param(4, &commainterval, fmt_int, 3);
 	x = fmt_advance();
 	check_type_integer(&x);
 	if (radix < 0 || radix > 36) {
 		vs_push(make_fixnum(radix));
 		FEerror("~D is illegal as a radix.", 1, vs_head);
 	}
-	fmt_integer(x, colon, atsign, radix, mincol, padchar, commachar);
+	fmt_integer(x, colon, atsign, radix, mincol, padchar, commachar, commainterval);
 }	
 
 static void
-fmt_integer(object x, bool colon, bool atsign, int radix, int mincol, int padchar, int commachar)
+fmt_integer(object x, bool colon, bool atsign, int radix, int mincol, int padchar, int commachar, int commainterval)
 {
 	int l, l1;
 	int s;
@@ -853,7 +863,7 @@ fmt_integer(object x, bool colon, bool atsign, int radix, int mincol, int padcha
 		writec_stream('+', fmt_stream);
 	while (l1-- > 0) {
 		writec_stream(fmt_tempstr(s++), fmt_stream);
-		if (colon && l1 > 0 && l1%3 == 0)
+		if (colon && l1 > 0 && l1%(commainterval) == 0)
 			writec_stream(commachar, fmt_stream);
 	}
 }
@@ -1016,7 +1026,7 @@ fmt_character(bool colon, bool atsign)
 static void
 fmt_fix_float(bool colon, bool atsign)
 {
-	int w, d, k, overflowchar, padchar;
+	int w=0, d=0, k=0, overflowchar=0, padchar=0;
 	double f;
 	int sign;
 	char buff[256], *b, buff1[256];
@@ -1208,7 +1218,7 @@ fmt_exponent1(int e)
 static void
 fmt_exponential_float(bool colon, bool atsign)
 {
-	int w, d, e, k, overflowchar, padchar, exponentchar;
+	int w=0, d=0, e=0, k=0, overflowchar=0, padchar=0, exponentchar=0;
 	double f;
 	int sign;
 	char buff[256], *b, buff1[256];
@@ -1412,7 +1422,7 @@ OVER:
 static void
 fmt_general_float(bool colon, bool atsign)
 {
-	int w, d, e, k, overflowchar, padchar, exponentchar;
+	int w=0, d=0, e=0, k, overflowchar, padchar=0, exponentchar;
 	int sign, exp;
 	char buff[256];
 	object x;
@@ -1504,7 +1514,7 @@ fmt_general_float(bool colon, bool atsign)
 static void
 fmt_dollars_float(bool colon, bool atsign)
 {
-	int d, n, w, padchar;
+	int d=0, n=0, w=0, padchar=0;
 	double f;
 	int sign;
 	char buff[256];
@@ -1591,7 +1601,7 @@ fmt_dollars_float(bool colon, bool atsign)
 static void
 fmt_percent(bool colon, bool atsign)
 {
-	int n, i;
+	int n=0, i;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1608,7 +1618,7 @@ fmt_percent(bool colon, bool atsign)
 static void
 fmt_ampersand(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1626,7 +1636,7 @@ fmt_ampersand(bool colon, bool atsign)
 static void
 fmt_bar(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1639,7 +1649,7 @@ fmt_bar(bool colon, bool atsign)
 static void
 fmt_tilde(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_set_param(0, &n, fmt_int, 1);
@@ -1667,7 +1677,7 @@ fmt_newline(bool colon, bool atsign)
 static void
 fmt_tabulate(bool colon, bool atsign)
 {
-	int colnum, colinc;
+	int colnum=0, colinc=0;
 	int c, i;
 	
 	fmt_max_param(2);
@@ -1703,7 +1713,7 @@ fmt_tabulate(bool colon, bool atsign)
 static void
 fmt_asterisk(bool colon, bool atsign)
 {
-	int n;
+	int n=0;
 
 	fmt_max_param(1);
 	fmt_not_colon_atsign(colon, atsign);
@@ -1793,42 +1803,46 @@ fmt_case(bool colon, bool atsign)
 	x = x->sm.sm_object0;
 	if (!colon && !atsign)
 		for (i = 0;  i < x->st.st_fillp;  i++) {
-			if (isUpper(j = x->st.st_self[i]))
-				j += 'a' - 'A';
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isUpper(j))
+		    j += 'a' - 'A';
+		  writec_stream(j, fmt_stream);
 		}
 	else if (colon && !atsign)
 		for (b = TRUE, i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i])) {
-				if (b)
-					j -= 'a' - 'A';
-				b = FALSE;
-			} else if (isUpper(j)) {
-				if (!b)
-					j += 'a' - 'A';
-				b = FALSE;
-			} else if (!isDigit(j))
-				b = TRUE;
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j)) {
+		    if (b)
+		      j -= 'a' - 'A';
+		    b = FALSE;
+		  } else if (isUpper(j)) {
+		    if (!b)
+		      j += 'a' - 'A';
+		    b = FALSE;
+		  } else if (!isDigit(j))
+		    b = TRUE;
+		  writec_stream(j, fmt_stream);
 		}
 	else if (!colon && atsign)
 		for (b = TRUE, i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i])) {
-				if (b)
-					j -= 'a' - 'A';
-				b = FALSE;
-			} else if (isUpper(j)) {
-				if (!b)
-					j += 'a' - 'A';
-				b = FALSE;
-			}
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j)) {
+		    if (b)
+		      j -= 'a' - 'A';
+		    b = FALSE;
+		  } else if (isUpper(j)) {
+		    if (!b)
+		      j += 'a' - 'A';
+		    b = FALSE;
+		  }
+		  writec_stream(j, fmt_stream);
 		}
 	else
 		for (i = 0;  i < x->st.st_fillp;  i++) {
-			if (isLower(j = x->st.st_self[i]))
-				j -= 'a' - 'A';
-			writec_stream(j, fmt_stream);
+		  j = x->st.st_self[i];
+		  if (isLower(j))
+		    j -= 'a' - 'A';
+		  writec_stream(j, fmt_stream);
 		}
 	vs_popp;
 	if (up_colon)
@@ -1840,7 +1854,7 @@ fmt_conditional(bool colon, bool atsign)
 {
 	int i, j, k;
 	object x;
-	int n;
+	int n=0;
 	bool done;
 	fmt_old;
 
@@ -1926,7 +1940,7 @@ fmt_conditional(bool colon, bool atsign)
 
 static void
 fmt_iteration(bool colon, bool atsign) {
-	int i,n;
+	int i,n=0;
 	VOL int j;
 	int o;
 	bool colon_close = FALSE;
@@ -2052,7 +2066,7 @@ fmt_iteration(bool colon, bool atsign) {
 static void
 fmt_justification(volatile bool colon, bool atsign)
 {
-	int mincol, colinc, minpad, padchar;
+	int mincol=0, colinc=0, minpad=0, padchar=0;
 	object fields[FORMAT_DIRECTIVE_LIMIT];
 	fmt_old;
 	jmp_buf fmt_jmp_buf0;
@@ -2178,7 +2192,7 @@ fmt_justification(volatile bool colon, bool atsign)
 static void
 fmt_up_and_out(bool colon, bool atsign)
 {
-	int i, j, k;
+	int i=0, j=0, k=0;
 
 	fmt_max_param(3);
 	fmt_not_atsign(atsign);
@@ -2215,6 +2229,8 @@ fmt_semicolon(bool colon, bool atsign)
 	fmt_set_param(1, &fmt_line_length, fmt_int, 72);
 }
 
+DEFVAR("*FORMAT-UNUSED-ARGS*",sSAformat_unused_argsA,SI,OBJNULL,"");
+
 DEFUNO_NEW("FORMAT",object,fLformat,LISP
        ,2,F_ARG_LIMIT,NONE,OO,OO,OO,OO,void,Lformat,(object strm, object control,...),"")
 {       va_list ap; 
@@ -2274,6 +2290,12 @@ DEFUNO_NEW("FORMAT",object,fLformat,LISP
 		    goto L;
 	    }
 	    format(strm, 0, control->st.st_fillp);
+	    if (sSAformat_unused_argsA->s.s_dbind) {
+	      int i;
+	      for (i=fmt_end-1;i>=fmt_index;i--)
+		sSAformat_unused_argsA->s.s_dbind=MMcons(fmt_base[i],sSAformat_unused_argsA->s.s_dbind);
+	    }
+
 	    flush_stream(strm);
 	}
 	    e = FALSE;
