@@ -53,7 +53,7 @@ static void
 mark_c_stack(jmp_buf, int, void (*)(void *,void *,int));
 
 static void
-mark_contblock(void *, int);
+mark_contblock(void *, fixnum);
 
 static void
 mark_object(object);
@@ -112,14 +112,14 @@ long first_protectable_page = 0;
 
 int runtime(void);
 
-static char *copy_relblock(char *p, int s);
+static char *copy_relblock(char *p, fixnum s);
 
 extern bool saving_system;
 extern long real_maxpage;
 extern long new_holepage;
 
-#define	available_pages	\
-	(real_maxpage-page(heap_end)-(new_holepage>=holepage ? new_holepage : holepage)-2*nrbpage-real_maxpage/32)
+/* #define	available_pages	\ */
+/* 	(real_maxpage-page(heap_end)-(new_holepage>=holepage ? new_holepage : holepage)-2*nrbpage-real_maxpage/32) */
 
 struct apage {
   char apage_self[PAGESIZE];
@@ -159,8 +159,6 @@ object sSAgbc_messageA;
 
 #endif
 
-#define	symbol_marked(x)	((x)->d.m)
-
 object *mark_origin[MARK_ORIGIN_MAX];
 int mark_origin_max;
 
@@ -180,14 +178,16 @@ enum type what_to_collect;
 void
 enter_mark_origin(object *p)
 {
-  unsigned long np=page(p);
 /*   if (np>=MAXPAGE) */
 /*     error("Address supplied to enter_mar_origin out of range"); */
   if (mark_origin_max >= MARK_ORIGIN_MAX)
     error("too many mark origins");
 #ifdef SGC
-  if (np<MAXPAGE)
-    sgc_type_map[np] |= SGC_PERM_WRITABLE ;
+ {
+   unsigned long np=page(p);
+   if (np<MAXPAGE)
+     sgc_type_map[np] |= SGC_PERM_WRITABLE ;
+ }
 #endif	
   mark_origin[mark_origin_max++] = p;
 }
@@ -200,32 +200,41 @@ enter_mark_origin(object *p)
 /*   mark_origin_block[mark_origin_block_max++].mob_size = n; */
 /* } */
 
+static void *mcsh,*mcsl;
+
 static void
 mark_cons(object x) {
   
   cs_check(x);
   
+#if CSTACK_DIRECTION == -1
+  if (mcsl>(void *)&x)
+#else
+  if (mcsl<(void *)&x)
+#endif
+    mcsl=&x;
+
   /*  x is already marked.  */
   
  BEGIN:  
   if (NULL_OR_ON_C_STACK(x->c.c_car)) goto MARK_CDR;
   if (type_of(x->c.c_car) == t_cons) {
-    if (x->c.c_car->c.m)
+    if (is_marked_or_free(x->c.c_car))
       ;
     else {
-      x->c.c_car->c.m = TRUE;
+      mark(x->c.c_car);
       mark_cons(x->c.c_car);
     }
   } else
     mark_object(x->c.c_car);
  MARK_CDR:  
-  x = x->c.c_cdr;
-  if (NULL_OR_ON_C_STACK(x))
+  if (NULL_OR_ON_C_STACK(x->c.c_cdr))
     return;
+  x = Scdr(x);
   if (type_of(x) == t_cons) {
-    if (x->c.m)
+    if (is_marked_or_free(x))
       return;
-    x->c.m = TRUE;
+    mark(x);
     goto BEGIN;
   }
   if (x == Cnil)
@@ -240,12 +249,18 @@ mark_cons(object x) {
 static void
 mark_object(object x) {
   
-  long i;
-  int j;
+  fixnum i,j;
   object *p;
   char *cp;
   
   cs_check(x);
+#if CSTACK_DIRECTION == -1
+  if (mcsl>(void *)&cp)
+#else
+  if (mcsl<(void *)&cp)
+#endif
+    mcsl=&cp;
+
  BEGIN:
   /* if the body of x is in the c stack, its elements
      are marked anyway by the c stack mark carefully, and
@@ -255,9 +270,9 @@ mark_object(object x) {
   
   if (NULL_OR_ON_C_STACK(x))
     return;
-  if (x->d.m)
+  if (is_marked_or_free(x))
     return;
-  x->d.m = TRUE;
+  mark(x);
   switch (type_of(x)) {
   case t_fixnum:
     break;
@@ -352,11 +367,11 @@ mark_object(object x) {
       if (inheap(x->a.a_dims)) {
 	if (what_to_collect == t_contiguous)
 	  mark_contblock((char *)(x->a.a_dims),
-			 sizeof(int)*x->a.a_rank);
+			 sizeof(fixnum)*x->a.a_rank);
       } else
-	x->a.a_dims = (int *)
+	x->a.a_dims = (fixnum *)
 	  copy_relblock((char *)(x->a.a_dims),
-			sizeof(int)*x->a.a_rank);
+			sizeof(fixnum)*x->a.a_rank);
     }
     if ((enum aelttype)x->a.a_elttype == aet_ch)
       goto CASE_STRING;
@@ -385,10 +400,12 @@ mark_object(object x) {
       break;
     case aet_char:
     case aet_uchar:
+    case aet_nnchar:
       j=sizeof(char)*x->a.a_dim;
       break;
     case aet_short:
     case aet_ushort:
+    case aet_nnshort:
       j=sizeof(short)*x->a.a_dim;
       break;
     default:
@@ -497,8 +514,8 @@ mark_object(object x) {
     /* We make bitvectors multiple of sizeof(int) in size allocated
        Assume 8 = number of bits in char */
     
-#define W_SIZE (8*sizeof(int))
-    j= sizeof(int) *
+#define W_SIZE (CHAR_SIZE*sizeof(fixnum))
+    j= sizeof(fixnum) *
       ((BV_OFFSET(x) + x->bv.bv_dim + W_SIZE -1)/W_SIZE);
     cp = x->bv.bv_self;
     if (cp == NULL)
@@ -579,8 +596,23 @@ mark_object(object x) {
       error("mark stream botch");
     }
     break;
-    
+
+#define MARK_CP(a_,b_) {fixnum _t=(b_);if (inheap(a_)) {\
+                           if (what_to_collect == t_contiguous) mark_contblock((void *)(a_),_t);\
+                        } else (a_)=(void *)copy_relblock((void *)(a_),_t);}
+
+#define MARK_MP(a_) {if ((a_)->_mp_d) \
+                        MARK_CP((a_)->_mp_d,(a_)->_mp_alloc*MP_LIMB_SIZE);}
+
   case t_random:
+    if ((int)what_to_collect >= (int)t_contiguous) {
+      MARK_MP(x->rnd.rnd_state._mp_seed);
+      if (x->rnd.rnd_state._mp_algdata._mp_lc) {
+	MARK_MP(x->rnd.rnd_state._mp_algdata._mp_lc->_mp_a);
+	if (!x->rnd.rnd_state._mp_algdata._mp_lc->_mp_m2exp) MARK_MP(x->rnd.rnd_state._mp_algdata._mp_lc->_mp_m);
+	MARK_CP(x->rnd.rnd_state._mp_algdata._mp_lc,sizeof(*x->rnd.rnd_state._mp_algdata._mp_lc));
+      }
+    }
     break;
     
   case t_readtable:
@@ -627,6 +659,10 @@ mark_object(object x) {
     mark_object(x->cf.cf_name);
     mark_object(x->cf.cf_data);
     break;
+
+  case t_ifun:
+    mark_object(x->ifn.ifn_self);
+    break;
     
   case t_cfdata:
     
@@ -638,7 +674,7 @@ mark_object(object x) {
       break;
     if (what_to_collect == t_contiguous) {
       if (!MAYBE_DATA_P((x->cfd.cfd_start)) ||
-	  get_mark_bit((long *)(x->cfd.cfd_start)))
+	  get_mark_bit(x->cfd.cfd_start))
 	break;
       mark_contblock(x->cfd.cfd_start, x->cfd.cfd_size);}
     break;
@@ -646,6 +682,7 @@ mark_object(object x) {
     mark_object(x->cc.cc_name);
     mark_object(x->cc.cc_env);
     mark_object(x->cc.cc_data);
+    if (x->cc.cc_turbo!=NULL) mark_object(*(x->cc.cc_turbo-1));
     if (what_to_collect == t_contiguous) {
       if (x->cc.cc_turbo != NULL)
 	mark_contblock((char *)(x->cc.cc_turbo-1),
@@ -669,7 +706,7 @@ static long *c_stack_where;
 static void
 mark_stack_carefully(void *topv, void *bottomv, int offset) {
 
-  long m,pageoffset;
+  long pageoffset;
   unsigned long p;
   object x;
   struct typemanager *tm;
@@ -699,14 +736,7 @@ mark_stack_carefully(void *topv, void *bottomv, int offset) {
 	 ((pageoffset=((char *)*j - pagetochar(p))) %
 	  tm->tm_size));
       if ((pageoffset <  (tm->tm_size * tm->tm_nppage))
-	  && (m=x->d.m) != FREE) {
-	if (m==TRUE) continue;
-	if (m!=0) {
-	  fprintf(stdout,
-		  "**bad value %ld of d.m in gbc page %ld skipping mark**"
-		  ,m,p);fflush(stdout);
-	  continue;
-	}
+	  && !is_marked_or_free(x)) {
 	mark_object(x);
       }
     }
@@ -893,9 +923,10 @@ mark_c_stack(jmp_buf env1, int n, void (*fn)(void *,void *,int)) {
   
 #if defined(__ia64__)
     {
-       extern void * __libc_ia64_register_backing_store_base;
+/*        extern void * __libc_ia64_register_backing_store_base; */
        void * bst=GC_save_regs_in_stack();
-       void * bsb=__libc_ia64_register_backing_store_base;
+/*        void * bsb=__libc_ia64_register_backing_store_base; */
+       void * bsb=cs_org2;
 
        if (bsb>bst)
           (*fn)(bsb,bst,C_GC_OFFSET);
@@ -916,8 +947,8 @@ sweep_phase(void) {
   STATIC struct typemanager *tm;
   STATIC object f;
   
-  Cnil->s.m = FALSE;
-  Ct->s.m = FALSE;
+  unmark(Cnil);
+  unmark(Ct);
   
 #ifdef DEBUG
   if (debug)
@@ -955,33 +986,20 @@ sweep_phase(void) {
     k = 0;
     for (j = tm->tm_nppage; j > 0; --j, p += tm->tm_size) {
       x = (object)p;
-      if (x->d.m == FREE)
+      if (is_free(x))
 	continue;
-      else if (x->d.m) {
-	x->d.m = FALSE;
+      else if (is_marked(x)) {
+	unmark(x);
 	continue;
       }
-      /*   Since we now mark forwards and backwards on displaced
-	   arrays, this is not necessary.
-	   switch (x->d.t) {
-	   case t_array:
-	   case t_vector:
-	   case t_string:
-	   case t_bitvector:
-	   if (x->a.a_displaced->c.c_car != Cnil)
-	   {undisplace(x);
-	   }
-	   }
-      */
-      /*			((struct freelist *)x)->f_link = f; */
       
 #ifdef GMP_USE_MALLOC
-      if (x->d.t == t_bignum) {
+      if (type_of(x) == t_bignum) {
 	mpz_clear(MP(x));
       }
 #endif
       SET_LINK(x,f);
-      x->d.m = FREE;
+      make_free(x);
       f = x;
       k++;
     }
@@ -1018,7 +1036,7 @@ contblock_sweep_phase(void) {
     s = pagetochar(i);
     e = pagetochar(j);
     for (p = s;  p < e;) {
-      if (get_mark_bit((int *)p)) {
+      if (get_mark_bit(p)) {
 	/* SGC cont pages: cont blocks must be no smaller than
 	   sizeof(struct contblock), and must not have a sweep
 	   granularity greater than this amount (e.g. CPTR_ALIGN) if
@@ -1029,7 +1047,7 @@ contblock_sweep_phase(void) {
       }
       q = p + CPTR_ALIGN;
       while (q < e) {
-	if (!get_mark_bit((int *)q)) {
+	if (!get_mark_bit(q)) {
 	  q += CPTR_ALIGN;
 	  continue;
 	}
@@ -1043,7 +1061,7 @@ contblock_sweep_phase(void) {
 #ifdef DEBUG
   if (debug) {
     for (cbp = cb_pointer; cbp != NULL; cbp = cbp->cb_link)
-      printf("%d-byte contblock\n", cbp->cb_size);
+      printf("%ld-byte contblock\n", cbp->cb_size);
     fflush(stdout);
   }
 #endif
@@ -1054,6 +1072,9 @@ int (*GBC_enter_hook)() = NULL;
 int (*GBC_exit_hook)() = NULL;
 char *old_rb_start;
 
+#define OPTIMIZE_MAX_PAGES (sSAoptimize_maximum_pagesA ==0 || sSAoptimize_maximum_pagesA->s.s_dbind !=sLnil) 
+#define IGNORE_MAX_PAGES (sSAignore_maximum_pagesA ==0 || sSAignore_maximum_pagesA->s.s_dbind !=sLnil) 
+
 void
 GBC(enum type t) {
 
@@ -1066,6 +1087,8 @@ GBC(enum type t) {
   int tm=0;
 #endif
   
+  mcsl=mcsh=&tm;
+
   if (in_signal_handler && t == t_relocatable)
     error("cant gc relocatable in signal handler");
   
@@ -1103,7 +1126,7 @@ GBC(enum type t) {
 	   (sgc_enabled ? sgc_count_type(t) : tm_of(t)->tm_npage),
 	   (tm_table[(int)t].tm_name)+1);
 #else
-    printf("[%s for %d %s pages..",
+    printf("[%s for %ld %s pages..",
 	   ("GC"),
 	   (tm_of(t)->tm_npage),
 	   (tm_table[(int)t].tm_name)+1);
@@ -1187,6 +1210,11 @@ GBC(enum type t) {
       i = (char *)&mark_table[j] - heap_end;
     else
       i = rb_end - heap_end;
+
+    if (i+heap_end>MAXCORE && !j) {
+      i=MAXCORE-heap_end;
+      printf("Warning: attempting tight relocatable GC\n");
+    }
     alloc_page(-(i + PAGESIZE - 1)/PAGESIZE);
     
     for (i = 0;  i < j; i++)
@@ -1333,6 +1361,11 @@ GBC(enum type t) {
 	exit(1);
       }
     core_end = heap_end + PAGESIZE;
+    {
+      unsigned long old_new_holepage=new_holepage;
+      new_holepage=CORE_PAGES/HOLEDIV;
+      if (available_pages<1) new_holepage=old_new_holepage;
+    }
     
 /*     for (i = 0;  i < maxpage;  i++) */
 /*       if ((enum type)type_map[i] == t_contiguous) */
@@ -1349,7 +1382,7 @@ GBC(enum type t) {
     if (sgc_enabled==0) 
 #endif
       {holepage = new_holepage;
-      nrbpage = INIT_NRBPAGE;}
+      nrbpage = CORE_PAGES/INIT_NRBDIV;}
     
     if (nrbpage < 0)
       error("no relocatable pages left");
@@ -1375,10 +1408,51 @@ GBC(enum type t) {
 
   }
   
+  {
+    extern int hole_overrun;
+
+    if (!hole_overrun && IGNORE_MAX_PAGES && OPTIMIZE_MAX_PAGES)
+      opt_maxpage(tm_table+t);
+
+  }
+
+  ZALLOCA(abs(mcsl-mcsh));
   
   CHECK_INTERRUPT;
+
+
 }
 
+static void
+FFN(siLheap_report)(void) {
+
+  int i;
+  
+  check_arg(0);
+  
+  vs_check_push(make_fixnum(sizeof(fixnum)*CHAR_SIZE));
+  vs_push(make_fixnum(PAGESIZE));
+  vs_push(make_fixnum(DBEGIN));
+  vs_push(make_fixnum(DBEGIN+(MAXPAGE<<PAGEWIDTH)));
+  vs_push(make_fixnum(SHARED_LIB_HEAP_CEILING));
+  i=sizeof(fixnum)*CHAR_SIZE-2;
+  i=1<<i;
+  vs_push(make_fixnum(((unsigned long)cs_base+i-1)&-i));
+  vs_push(make_fixnum(abs(cs_base-cs_org)));
+  vs_push(make_fixnum((CSTACK_DIRECTION+1)>>1));
+  vs_push(make_fixnum(CSTACK_ALIGNMENT));
+  vs_push(make_fixnum(CSSIZE));
+#if defined(IM_FIX_BASE) && defined(IM_FIX_LIM)
+  vs_push(make_fixnum(IM_FIX_BASE));
+  vs_push(make_fixnum(IM_FIX_LIM));
+#else  
+  vs_push(Cnil);
+  vs_push(Cnil);
+#endif
+
+}  
+
+
 static void
 FFN(siLroom_report)(void) {
 
@@ -1408,12 +1482,14 @@ FFN(siLroom_report)(void) {
       vs_push(make_fixnum(tm_table[i].tm_npage));
       vs_push(make_fixnum(tm_table[i].tm_maxpage));
       vs_push(make_fixnum(tm_table[i].tm_gbccount));
+      vs_push(make_fixnum(tm_table[i].tm_size/sizeof(fixnum)));
     } else {
       vs_check_push(Cnil);
       vs_push(make_fixnum(tm_table[i].tm_type));
       vs_push(Cnil);
       vs_push(Cnil);
       vs_push(Cnil);
+      vs_push(Cnil);
     }
   }
 }
@@ -1436,12 +1512,14 @@ FFN(siLreset_gbc_count)(void) {
 */
 
 static char *
-copy_relblock(char *p, int s)
+copy_relblock(char *p, fixnum s)
 { char *res = rb_pointer;
  char *q = rb_pointer1;
  s = ROUND_UP_PTR(s);
  rb_pointer += s;
  rb_pointer1 += s;
+ if (rb_pointer1>core_end)
+   error("not enough room to gc relblock");
  
  while (--s >= 0)
    { *q++ = *p++;}
@@ -1451,7 +1529,7 @@ copy_relblock(char *p, int s)
 
 
 static void
-mark_contblock(void *p, int s) {
+mark_contblock(void *p, fixnum s) {
 
   STATIC char *q;
   STATIC char *x, *y;
@@ -1506,6 +1584,7 @@ void
 gcl_init_GBC(void) {
 
   make_si_function("ROOM-REPORT", siLroom_report);
+  make_si_function("HEAP-REPORT", siLheap_report);
   make_si_function("RESET-GBC-COUNT", siLreset_gbc_count);
   make_si_function("GBC-TIME",siLgbc_time);
   
