@@ -62,6 +62,32 @@ object fixnum_sub(fixnum i, fixnum j)
    }
 }
 
+object 
+fixnum_times(fixnum i, fixnum j) {
+
+  if (i>=0) {
+    if (j>=0) {
+      if (!i || j<= (MOST_POSITIVE_FIX/i))
+	goto FIX;
+    } else {
+      if (j==-1 || i<= (MOST_NEGATIVE_FIX/j))
+	goto FIX;
+    }
+  } else {
+    if (j>=0) {
+      if (i==-1 || j<= (MOST_NEGATIVE_FIX/i))
+	goto FIX;
+    } else {
+      if (0<-i && -i<= (MOST_POSITIVE_FIX/-j))
+	goto FIX;
+    }
+  }
+  MPOP(return,mulss,i,j);
+ FIX:
+  return make_fixnum(i*j);
+}
+
+/* return i - j */
 /* static object */
 /* fixnum_times(int i, int j) */
 /* { */
@@ -103,9 +129,10 @@ integer_exact_quotient(object x,object y,int in_place) {
   if (y==small_fixnum(1) || x==small_fixnum(0))
     return x;
 
-  if (type_of(y)==t_fixnum && type_of(x)==t_fixnum) 
-    return make_fixnum(fix(x)/fix(y)); /* no in_place for fixnums as could be small */;
-
+  if (type_of(x)==t_fixnum)  /* no in_place for fixnums as could be small */
+    return make_fixnum((type_of(y)==t_fixnum ? fix(x)/fix(y) : -1)); 
+  /* Only big dividing a fix is most-negative-fix/abs(most-negative-fix)*/
+    
   r=in_place ? x : new_bignum();
 
   if (type_of(y)==t_fixnum) 
@@ -589,7 +616,8 @@ number_times(object x, object y)
 	case t_fixnum:
 		switch (type_of(y)) {
 		case t_fixnum:
-		  MPOP(return,mulss,fix(x),fix(y));
+		  return fixnum_times(fix(x),fix(y));
+/* 		  MPOP(return,mulss,fix(x),fix(y)); */
 		case t_bignum:
 		  MPOP(return,mulsi,fix(x),MP(y));
 		case t_ratio:
