@@ -225,7 +225,7 @@ truename(object pathname)
 	char truefilename[MAXPATHLEN];
 	char current_directory[MAXPATHLEN];
 	char directory[MAXPATHLEN];
-	static char *getwd(char *buffer);
+	char *getwd(char *buffer);
 	int islinkcount=8;
 	struct stat filestatus;
 
@@ -247,7 +247,7 @@ truename(object pathname)
 		return(file_error("Symlink broken at ~S.",pathname));
 	    for (p = filename, q = 0;  *p != '\0';  p++)
 		    if (*p == '/') q = p;
-	    if (q == 0)
+	    if (q == 0 || *newname == '/')
 		q = filename;
 	    else
 		q++;
@@ -372,7 +372,7 @@ file_exists(object file)
         }
 #endif        
 
-	if (stat(filename, &filestatus) >= 0) {
+	if (stat(filename, &filestatus) >= 0  && !S_ISDIR(filestatus.st_mode)) {
 #ifdef AIX
 	    /* if /tmp/foo is not a directory /tmp/foo/ should not exist */
 	    if (filename[strlen(filename)-1] == '/' &&
@@ -383,12 +383,24 @@ file_exists(object file)
 	} else
 	if (sSAallow_gzipped_fileA->s.s_dbind != sLnil
 	      && (strcat(filename,".gz"),
-		  stat(filename, &filestatus) >= 0))
+		  stat(filename, &filestatus) >= 0 && !S_ISDIR(filestatus.st_mode)))
 	    return TRUE;
 	else
 	    return(FALSE);
 }
 
+FILE *
+fopen_not_dir(char *filename,char * option) {
+
+  struct stat ss;
+
+  if (!stat(filename,&ss) && S_ISDIR(ss.st_mode))
+    return NULL;
+  else
+    return fopen(filename,option);
+
+}
+
 FILE *
 backup_fopen(char *filename, char *option)
 {
@@ -431,12 +443,8 @@ LFD(Ltruename)(void)
 	    !file_exists(vs_base[0])) {
 	    file_error("File ~S does not exist.",vs_base[0]);
 	    vs_base[0] = Cnil;
-	} else {
-	    if ((vs_base[0]->pn.pn_directory == Cnil) || 
-		    (vs_base[0]->pn.pn_directory == sKunspecific))
-		vs_base[0]->pn.pn_directory=make_cons(sKrelative,Cnil);
+	} else 
 	    vs_base[0] = truename(vs_base[0]);
-	}
 }
 
 LFD(Lrename_file)(void)
@@ -562,7 +570,8 @@ LFD(Lfile_write_date)(void)
 
 
 	coerce_to_local_filename(vs_base[0], filename);
-	if (stat(filename, &filestatus) < 0) { vs_base[0] = Cnil; return;}
+	if (stat(filename, &filestatus) < 0 || S_ISDIR(filestatus.st_mode)) 
+	  { vs_base[0] = Cnil; return;}
 	vs_base[0] = unix_time_to_universal_time(filestatus.st_mtime);
 }
 
@@ -586,7 +595,8 @@ LFD(Lfile_author)(void)
 	}
 
 	coerce_to_local_filename(vs_base[0], filename);
-	if (stat(filename, &filestatus) < 0) { vs_base[0] = Cnil; return;}
+	if (stat(filename, &filestatus) < 0 || S_ISDIR(filestatus.st_mode)) 
+	  { vs_base[0] = Cnil; return;}
 	pwent = getpwuid(filestatus.st_uid);
 	vs_base[0] = pwent ? make_simple_string(pwent->pw_name) : Cnil;
 #else
